@@ -1,3 +1,4 @@
+/* $XFree86: xc/extras/freetype2/src/type1/t1load.c,v 1.2 2007/04/05 16:08:58 tsi Exp $ */
 /***************************************************************************/
 /*                                                                         */
 /*  t1load.c                                                               */
@@ -1783,6 +1784,9 @@
     if ( error )
       goto Exit;
 
+    /* Ensure num_blue_values is even */
+    priv->num_blue_values &= ~1;
+
 #ifndef T1_CONFIG_OPTION_NO_MM_SUPPORT
 
     /* the following can happen for MM instances; we then treat the */
