@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_gx2_driver.c,v 1.16 2006/03/16 16:50:11 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_gx2_driver.c,v 1.19 2007/01/23 18:03:05 tsi Exp $ */
 /*
  * File Contents: This is the main module configures the interfacing 
  *                with the X server. The individual modules will be 
@@ -139,7 +139,6 @@
  *
  * END_NSC_LIC_GPL */
 
-#define DEBUG(x)
 #define GEODE_TRACE 0
 
 /* Includes that are used by all drivers */
@@ -204,7 +203,7 @@ extern OptionInfoRec GeodeOptions[];
 
 /* Forward definitions */
 static Bool GX2PreInit(ScrnInfoPtr, int);
-static Bool GX2ScreenInit(int, ScreenPtr, int, char **);
+static Bool GX2ScreenInit(int, ScreenPtr, const int, const char **);
 static Bool GX2EnterVT(int, int);
 static void GX2LeaveVT(int, int);
 static void GX2FreeScreen(int, int);
@@ -1696,7 +1695,8 @@ GX2DPMSSet(ScrnInfoPtr pScreenInfo, int mode, int flags)
 *----------------------------------------------------------------------------
 */
 static Bool
-GX2ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+GX2ScreenInit(int scrnIndex, ScreenPtr pScreen,
+	      const int argc, const char **argv)
 {
    ScrnInfoPtr pScreenInfo = xf86Screens[pScreen->myNum];
    GeodePtr pGeode;
