@@ -1,17 +1,17 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/vbe/vbe.c,v 1.6 2005/02/26 03:18:38 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/vbe/vbe.c,v 1.19 2006/03/16 16:50:35 dawes Exp $ */
 
 /*
  *                   XFree86 vbe module
  *               Copyright 2000 Egbert Eich
  *
- * The mode query/save/set/restore functions from the vesa driver 
+ * The mode query/save/set/restore functions from the vesa driver
  * have been moved here.
  * Copyright (c) 2000 by Conectiva S.A. (http://www.conectiva.com)
- * Authors: Paulo Csar Pereira de Andrade <pcpa@conectiva.com.br> 
+ * Authors: Paulo Csar Pereira de Andrade <pcpa@conectiva.com.br>
  */
 
 /*
- * Copyright (c) 2002-2005 by The XFree86 Project, Inc.
+ * Copyright (c) 2002-2006 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -57,26 +57,16 @@
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "xf86.h"
-#include "xf86_ansic.h"
 #include "vbe.h"
-#include "Xarch.h"
+#include <X11/Xarch.h>
 #define DPMS_SERVER
-#include "extensions/dpms.h"
+#include <X11/extensions/dpms.h>
 
-#define VERSION(x) VBE_VERSION_MAJOR(x),VBE_VERSION_MINOR(x)
+#define VERSION(x) VBE_VERSION_MAJOR(x), VBE_VERSION_MINOR(x)
 
-#if X_BYTE_ORDER == X_LITTLE_ENDIAN
-#define B_O16(x)  (x) 
-#define B_O32(x)  (x)
-#else
-#define B_O16(x)  ((((x) & 0xff) << 8) | (((x) & 0xff) >> 8))
-#define B_O32(x)  ((((x) & 0xff) << 24) | (((x) & 0xff00) << 8) \
-                  | (((x) & 0xff0000) >> 8) | (((x) & 0xff000000) >> 24))
-#endif
 #define L_ADD(x)  (B_O32(x) & 0xffff) + ((B_O32(x) >> 12) & 0xffff00)
 
-#define FARP(p)		(((unsigned)(p & 0xffff0000) >> 12) | (p & 0xffff))
+#define FARP(p)		((((unsigned)p & 0xffff0000) >> 12) + (p & 0xffff))
 #define R16(v)		((v) & 0xffff)
 
 static unsigned char * vbeReadEDID(vbeInfoPtr pVbe);
@@ -88,6 +78,16 @@ const char *vbe_ddcSymbols[] = {
     NULL
 };
 
+const char *vbe_int10Symbols[] = {
+    "xf86ExecX86int10",
+    "xf86FreeInt10",
+    "xf86InitInt10",
+    "xf86Int10AllocPages",
+    "xf86Int10FreePages",
+    "xf86int10Addr",
+    NULL
+};
+
 static const char vbeVersionString[] = "VBE2";
 
 vbeInfoPtr
@@ -106,83 +106,96 @@ VBEExtendedInit(xf86Int10InfoPtr pInt, int entityIndex, int Flags)
     Bool init_int10 = FALSE;
     vbeInfoPtr vip = NULL;
     int screen = pScrn->scrnIndex;
+    unsigned long TotalMem;
+    ModuleDescPtr pMod, pInt10Mod = NULL;
 
     if (!pInt) {
-	if (!xf86LoadSubModule(pScrn, "int10"))
+	if (!(pInt10Mod = xf86LoadSubModule(pScrn, "int10")))
 	    goto error;
 
-	xf86DrvMsg(screen,X_INFO,"initializing int10\n");
+	xf86LoaderModReqSymLists(pInt10Mod, vbe_int10Symbols, NULL);
+	xf86DrvMsg(screen, X_INFO, "initializing int10\n");
 	pInt = xf86InitInt10(entityIndex);
 	if (!pInt)
 	    goto error;
 	init_int10 = TRUE;
+    } else {
+	if ((pMod = xf86GetSubModuleByName(pScrn->module, "int10")))
+	    xf86LoaderModReqSymLists(pMod, vbe_int10Symbols, NULL);
     }
-    
-    page = xf86Int10AllocPages(pInt,1,&RealOff);
+
+    page = xf86Int10AllocPages(pInt, 1, &RealOff);
     if (!page) goto error;
-    vbe = (vbeControllerInfoPtr) page;    
-    memcpy(vbe->VbeSignature,vbeVersionString,4);
+    vbe = (vbeControllerInfoPtr) page;
+    (void) memset(vbe, 0, 512);
+    (void) memcpy(vbe->VbeSignature, vbeVersionString, 4);
 
     pInt->ax = 0x4F00;
     pInt->es = SEG_ADDR(RealOff);
     pInt->di = SEG_OFF(RealOff);
     pInt->num = 0x10;
-    
+
     xf86ExecX86int10(pInt);
 
     if ((pInt->ax & 0xff) != 0x4f) {
-	xf86DrvMsgVerb(screen,X_INFO,3,"VESA BIOS not detected\n");
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA BIOS not detected\n");
 	goto error;
     }
-    
+
     switch (pInt->ax & 0xff00) {
     case 0:
-	xf86DrvMsg(screen,X_INFO,"VESA BIOS detected\n");
+	xf86DrvMsg(screen, X_INFO, "VESA BIOS detected\n");
 	break;
     case 0x100:
-	xf86DrvMsg(screen,X_INFO,"VESA BIOS function failed\n");
+	xf86DrvMsg(screen, X_INFO, "VESA BIOS function failed\n");
 	goto error;
     case 0x200:
-	xf86DrvMsg(screen,X_INFO,"VESA BIOS not supported\n");
+	xf86DrvMsg(screen, X_INFO, "VESA BIOS not supported\n");
 	goto error;
     case 0x300:
-	xf86DrvMsg(screen,X_INFO,"VESA BIOS not supported in current mode\n");
+	xf86DrvMsg(screen, X_INFO, "VESA BIOS not supported in current mode\n");
 	goto error;
     default:
-	xf86DrvMsg(screen,X_INFO,"Invalid\n");
+	xf86DrvMsg(screen, X_INFO, "Invalid\n");
 	goto error;
     }
-    
+
     xf86DrvMsgVerb(screen, X_INFO, 4,
-		"VbeVersion is %d, OemStringPtr is 0x%08lx,\n"
+		"VbeVersion is %ld, OemStringPtr is 0x%08lx,\n"
 		"\tOemVendorNamePtr is 0x%08lx, OemProductNamePtr is 0x%08lx,\n"
 		"\tOemProductRevPtr is 0x%08lx\n",
-		vbe->VbeVersion, (unsigned long)vbe->OemStringPtr,
-		(unsigned long)vbe->OemVendorNamePtr,
-		(unsigned long)vbe->OemProductNamePtr,
-		(unsigned long)vbe->OemProductRevPtr);
+		(unsigned long)B_O16(vbe->VbeVersion),
+		(unsigned long)B_O32(vbe->OemStringPtr),
+		(unsigned long)B_O32(vbe->OemVendorNamePtr),
+		(unsigned long)B_O32(vbe->OemProductNamePtr),
+		(unsigned long)B_O32(vbe->OemProductRevPtr));
 
-    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE Version %i.%i\n",
+    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE Version %i.%i\n",
 		   VERSION(vbe->VbeVersion));
-    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE Total Mem: %i kB\n",
-		   vbe->TotalMem * 64);
-    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE OEM: %s\n",
-		   (CARD8*)xf86int10Addr(pInt,L_ADD(vbe->OemStringPtr)));
-    
+
+    TotalMem = B_O16(vbe->TotalMem);
+    /* Some BIOS'es are off by one */
+    TotalMem += TotalMem & 1U;
+
+    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE Total Mem: %li kB\n",
+		   TotalMem * 64);
+    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE OEM: %s\n",
+		   (CARD8*)xf86int10Addr(pInt, L_ADD(vbe->OemStringPtr)));
+
     if (B_O16(vbe->VbeVersion) >= 0x200) {
-	xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE OEM Software Rev: %i.%i\n",
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE OEM Software Rev: %i.%i\n",
 		    VERSION(vbe->OemSoftwareRev));
 	if (vbe->OemVendorNamePtr)
-	    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE OEM Vendor: %s\n",
-		    (CARD8*)xf86int10Addr(pInt,L_ADD(vbe->OemVendorNamePtr)));
+	    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE OEM Vendor: %s\n",
+		    (CARD8*)xf86int10Addr(pInt, L_ADD(vbe->OemVendorNamePtr)));
 	if (vbe->OemProductNamePtr)
-	    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE OEM Product: %s\n",
-		    (CARD8*)xf86int10Addr(pInt,L_ADD(vbe->OemProductNamePtr)));
+	    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE OEM Product: %s\n",
+		    (CARD8*)xf86int10Addr(pInt, L_ADD(vbe->OemProductNamePtr)));
 	if (vbe->OemProductRevPtr)
-	    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE OEM Product Rev: %s\n",
-		    (CARD8*)xf86int10Addr(pInt,L_ADD(vbe->OemProductRevPtr)));
+	    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE OEM Product Rev: %s\n",
+		    (CARD8*)xf86int10Addr(pInt, L_ADD(vbe->OemProductRevPtr)));
     }
-    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE Capabilities: "
+    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE Capabilities: "
 		   "0x%02x%02x%02x%02x\n",
 		   vbe->Capabilities[3], vbe->Capabilities[2],
 		   vbe->Capabilities[1], vbe->Capabilities[0]);
@@ -194,14 +207,19 @@ VBEExtendedInit(xf86Int10InfoPtr pInt, int entityIndex, int Flags)
     vip->real_mode_base = RealOff;
     vip->num_pages = 1;
     vip->init_int10 = init_int10;
+    vip->pInt10Mod = pInt10Mod;
+    vip->pDDCMod = NULL;
 
     return vip;
 
  error:
     if (page)
 	xf86Int10FreePages(pInt, page, 1);
-    if (init_int10)
+    if (init_int10) {
 	xf86FreeInt10(pInt);
+	if (pInt10Mod)
+	    xf86UnloadSubModule(pInt10Mod);
+    }
     return NULL;
 }
 
@@ -211,10 +229,15 @@ vbeFree(vbeInfoPtr pVbe)
     if (!pVbe)
 	return;
 
-    xf86Int10FreePages(pVbe->pInt10,pVbe->memory,pVbe->num_pages);
+    xf86Int10FreePages(pVbe->pInt10, pVbe->memory, pVbe->num_pages);
     /* If we have initalized int10 we ought to free it, too */
-    if (pVbe->init_int10) 
+    if (pVbe->init_int10) {
 	xf86FreeInt10(pVbe->pInt10);
+	if (pVbe->pInt10Mod)
+	    xf86UnloadSubModule(pVbe->pInt10Mod);
+    }
+    if (pVbe->pDDCMod)
+	xf86UnloadSubModule(pVbe->pDDCMod);
     xfree(pVbe);
     return;
 }
@@ -224,7 +247,7 @@ vbeProbeDDC(vbeInfoPtr pVbe)
 {
     char *ddc_level;
     int screen = pVbe->pInt10->scrnIndex;
-    
+
     if (!pVbe || (pVbe->ddc == DDC_NONE))
 	return FALSE;
     if (pVbe->ddc != DDC_UNCHECKED)
@@ -240,50 +263,53 @@ vbeProbeDDC(vbeInfoPtr pVbe)
     xf86ExecX86int10(pVbe->pInt10);
 
     if ((pVbe->pInt10->ax & 0xff) != 0x4f) {
-        xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE DDC not supported\n");
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE DDC not supported\n");
 	pVbe->ddc = DDC_NONE;
 	return FALSE;
     }
 
     switch ((pVbe->pInt10->ax >> 8) & 0xff) {
     case 0:
-	xf86DrvMsg(screen,X_INFO,"VESA VBE DDC supported\n");
+	xf86DrvMsg(screen, X_INFO, "VESA VBE DDC supported\n");
 	switch (pVbe->pInt10->bx & 0x3) {
 	case 0:
-  	    ddc_level = " none"; 
+	    ddc_level = " none";
 	    pVbe->ddc = DDC_NONE;
 	    break;
 	case 1:
-  	    ddc_level = " 1";
+	    ddc_level = " 1";
 	    pVbe->ddc = DDC_1;
 	    break;
 	case 2:
-  	    ddc_level = " 2"; 
+	    ddc_level = " 2";
 	    pVbe->ddc = DDC_2;
 	    break;
 	case 3:
-  	    ddc_level = " 1 + 2"; 
+	    ddc_level = " 1 + 2";
 	    pVbe->ddc = DDC_1_2;
 	    break;
 	default:
- 	    ddc_level = "";
+	    ddc_level = "";
 	    pVbe->ddc = DDC_NONE;
 	    break;
 	}
-  	xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE DDC Level%s\n",ddc_level); 
-  	if (pVbe->pInt10->bx & 0x4) {
-    	    xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE DDC Screen blanked" 
-    			"for data transfer\n"); 
-    	    pVbe->ddc_blank = TRUE;
-    	}  else
-    	    pVbe->ddc_blank = FALSE;
-	    
-  	xf86DrvMsgVerb(screen,X_INFO,3,
-		       "VESA VBE DDC transfer in appr. %x sec.\n", 
-		       (pVbe->pInt10->bx >> 8) & 0xff); 
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE DDC Level%s\n", ddc_level);
+	if (pVbe->ddc == DDC_NONE)
+	    return FALSE;
+	if (pVbe->pInt10->bx & 0x4) {
+	    xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE DDC Screen blanked"
+			"for data transfer\n");
+	    pVbe->ddc_blank = TRUE;
+	} else {
+	    pVbe->ddc_blank = FALSE;
+	}
+
+	xf86DrvMsgVerb(screen, X_INFO, 3,
+		       "VESA VBE DDC transfer in approximately %x second(s)\n",
+		       (pVbe->pInt10->bx >> 8) & 0xff);
     }
-    
-    return TRUE; 
+
+    return TRUE;
 }
 
 typedef enum {
@@ -314,23 +340,23 @@ vbeReadEDID(vbeInfoPtr pVbe)
     if (!page) return NULL;
 
     options = xnfalloc(sizeof(VBEOptions));
-    (void)memcpy(options, VBEOptions, sizeof(VBEOptions));
+    (void) memcpy(options, VBEOptions, sizeof(VBEOptions));
     xf86ProcessOptions(screen, xf86Screens[screen]->options, options);
     xf86GetOptValBool(options, VBEOPT_NOVBE, &novbe);
     xf86GetOptValBool(options, VBEOPT_NODDC, &noddc);
     edidOption = xf86GetOptValString(options, VBEOPT_EDID_DATA);
     xfree(options);
     if (novbe || noddc) return NULL;
-    
+
     if (edidOption)
 	tmp = xf86DoEDID_Option(screen, edidOption);
     if (tmp)
 	return tmp;
-	
+
     if (!vbeProbeDDC(pVbe)) goto error;
 
-    memset(page,0,sizeof(vbeInfoPtr));
-    strcpy(page,vbeVersionString);
+    (void) memset(page, 0, sizeof(vbeInfoPtr));
+    (void) memcpy(page, vbeVersionString, 4);
 
     pVbe->pInt10->ax = 0x4F15;
     pVbe->pInt10->bx = 0x01;
@@ -343,71 +369,61 @@ vbeReadEDID(vbeInfoPtr pVbe)
     xf86ExecX86int10(pVbe->pInt10);
 
     if ((pVbe->pInt10->ax & 0xff) != 0x4f) {
-        xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE DDC invalid\n");
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE DDC invalid\n");
 	goto error;
     }
     switch (pVbe->pInt10->ax & 0xff00) {
     case 0x0:
-	xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE DDC read successfully\n");
-  	tmp = (unsigned char *)xnfalloc(128); 
-  	memcpy(tmp,page,128); 
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE DDC read successfully\n");
+	tmp = (unsigned char *)xnfalloc(128);
+	(void) memcpy(tmp, page, 128);
 	break;
     case 0x100:
-	xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE DDC read failed\n");	
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE DDC read failed\n");
 	break;
     default:
-	xf86DrvMsgVerb(screen,X_INFO,3,"VESA VBE DDC unkown failure %i\n",
+	xf86DrvMsgVerb(screen, X_INFO, 3, "VESA VBE DDC unknown failure %i\n",
 		       pVbe->pInt10->ax & 0xff00);
 	break;
     }
-    
+
  error:
     return tmp;
 }
 
 xf86MonPtr
-vbeDoEDID(vbeInfoPtr pVbe, pointer pDDCModule)
+vbeDoEDID(vbeInfoPtr pVbe, ModuleDescPtr pDDCModule)
 {
     xf86MonPtr    pMonitor;
-    pointer       pModule;
+    ModuleDescPtr pModule;
     unsigned char *DDC_data = NULL;
-    
+
     if (!pVbe) return NULL;
     if (pVbe->version < 0x200)
 	return NULL;
 
     if (!(pModule = pDDCModule)) {
-	pModule =
+	pVbe->pDDCMod = pModule =
 	    xf86LoadSubModule(xf86Screens[pVbe->pInt10->scrnIndex], "ddc");
 	if (!pModule)
 	    return NULL;
 
-	xf86LoaderReqSymLists(vbe_ddcSymbols, NULL);
     }
-        
+    xf86LoaderModReqSymLists(pModule, vbe_ddcSymbols, NULL);
+
     DDC_data = vbeReadEDID(pVbe);
 
-    if (!DDC_data) 
+    if (!DDC_data)
 	return NULL;
-    
+
     pMonitor = xf86InterpretEDID(pVbe->pInt10->scrnIndex, DDC_data);
 
-    if (!pDDCModule)
-        xf86UnloadSubModule(pModule);
     return pMonitor;
 }
 
-
 VbeInfoBlock *
 VBEGetVBEInfo(vbeInfoPtr pVbe)
 {
-    VbeInfoBlock *block = NULL;
-    int i, pStr, pModes;
-    char *str;
-    CARD16 major, *modes;
-
-    bzero(pVbe->memory, sizeof(VbeInfoBlock));
-
     /*
     Input:
 	AH    := 4Fh	Super VGA support
@@ -419,10 +435,13 @@ VBEGetVBEInfo(vbeInfoPtr pVbe)
 	(All other registers are preserved)
      */
 
-    ((char*)pVbe->memory)[0] = 'V';
-    ((char*)pVbe->memory)[1] = 'B';
-    ((char*)pVbe->memory)[2] = 'E';
-    ((char*)pVbe->memory)[3] = '2';
+    VbeInfoBlock *block = NULL;
+    int i, pStr, pModes;
+    char *str, *pBlock = pVbe->memory;
+    CARD16 *modes;
+
+    (void) memset(pBlock, 0, 512);
+    (void) memcpy(pBlock, vbeVersionString, 4);
 
     pVbe->pInt10->num = 0x10;
     pVbe->pInt10->ax = 0x4f00;
@@ -431,60 +450,73 @@ VBEGetVBEInfo(vbeInfoPtr pVbe)
     xf86ExecX86int10(pVbe->pInt10);
 
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (NULL);
+	return NULL;
 
-    block = xcalloc(sizeof(VbeInfoBlock), 1);
-    block->VESASignature[0] = ((char*)pVbe->memory)[0];
-    block->VESASignature[1] = ((char*)pVbe->memory)[1];
-    block->VESASignature[2] = ((char*)pVbe->memory)[2];
-    block->VESASignature[3] = ((char*)pVbe->memory)[3];
+    block = xnfcalloc(sizeof(VbeInfoBlock), 1);
+    block->VESASignature[0] = pBlock[0];
+    block->VESASignature[1] = pBlock[1];
+    block->VESASignature[2] = pBlock[2];
+    block->VESASignature[3] = pBlock[3];
 
-    block->VESAVersion = *(CARD16*)(((char*)pVbe->memory) + 4);
-    major = (unsigned)block->VESAVersion >> 8;
+    block->VESAVersion = B_O16(pBlock[4]);
 
-    pStr = *(CARD32*)(((char*)pVbe->memory) + 6);
+    pStr = B_O32(pBlock[6]);
     str = xf86int10Addr(pVbe->pInt10, FARP(pStr));
     block->OEMStringPtr = strdup(str);
 
-    block->Capabilities[0] = ((char*)pVbe->memory)[10];
-    block->Capabilities[1] = ((char*)pVbe->memory)[11];
-    block->Capabilities[2] = ((char*)pVbe->memory)[12];
-    block->Capabilities[3] = ((char*)pVbe->memory)[13];
+    block->Capabilities[0] = pBlock[10];
+    block->Capabilities[1] = pBlock[11];
+    block->Capabilities[2] = pBlock[12];
+    block->Capabilities[3] = pBlock[13];
 
-    pModes = *(CARD32*)(((char*)pVbe->memory) + 14);
+    /*
+     * XXX This doesn't check if the mode list overflows.
+     */
+    pModes = B_O32(pBlock[14]);
     modes = xf86int10Addr(pVbe->pInt10, FARP(pModes));
-    i = 0;
-    while (modes[i] != 0xffff)
-	i++;
-    block->VideoModePtr = xalloc(sizeof(CARD16) * i + 1);
-    memcpy(block->VideoModePtr, modes, sizeof(CARD16) * i);
+    for (i = 0;  modes[i] != 0xffff;  i++);
+    block->VideoModePtr = xnfalloc(sizeof(CARD16) * i + 1);
+    for (i = 0;  modes[i] != 0xffff;  i++)
+	block->VideoModePtr[i] = B_O16(modes[i]);
     block->VideoModePtr[i] = 0xffff;
 
-    block->TotalMemory = *(CARD16*)(((char*)pVbe->memory) + 18);
+    block->TotalMemory = B_O16(pBlock[18]);
+    block->TotalMemory += block->TotalMemory & 1U;
 
-    if (major < 2)
-	memcpy(&block->OemSoftwareRev, ((char*)pVbe->memory) + 20, 236);
-    else {
-	block->OemSoftwareRev = *(CARD16*)(((char*)pVbe->memory) + 20);
-	pStr = *(CARD32*)(((char*)pVbe->memory) + 22);
+    if (block->VESAVersion < 0x0200) {
+	(void) memcpy(&block->Reserved, pBlock + 20, 236);
+    } else {
+	block->OemSoftwareRev = B_O16(pBlock[20]);
+
+	/*
+	 * XXX Consider using strndup for these three.
+	 */
+
+	pStr = B_O32(pBlock[22]);
 	str = xf86int10Addr(pVbe->pInt10, FARP(pStr));
 	block->OemVendorNamePtr = strdup(str);
-	pStr = *(CARD32*)(((char*)pVbe->memory) + 26);
+
+	pStr = B_O32(pBlock[26]);
 	str = xf86int10Addr(pVbe->pInt10, FARP(pStr));
 	block->OemProductNamePtr = strdup(str);
-	pStr = *(CARD32*)(((char*)pVbe->memory) + 30);
+
+	pStr = B_O32(pBlock[30]);
 	str = xf86int10Addr(pVbe->pInt10, FARP(pStr));
 	block->OemProductRevPtr = strdup(str);
-	memcpy(&block->Reserved, ((char*)pVbe->memory) + 34, 222);
-	memcpy(&block->OemData, ((char*)pVbe->memory) + 256, 256);
+
+	(void) memcpy(&block->Reserved, pBlock + 34, 222);
+	(void) memcpy(&block->OemData, pBlock + 256, 256);
     }
 
-    return (block);
+    return block;
 }
 
 void
 VBEFreeVBEInfo(VbeInfoBlock *block)
 {
+    if (!block)
+	return;
+
     xfree(block->OEMStringPtr);
     xfree(block->VideoModePtr);
     if (((unsigned)block->VESAVersion >> 8) >= 2) {
@@ -517,19 +549,49 @@ VBESetVBEMode(vbeInfoPtr pVbe, int mode, VbeCRTCInfoBlock *block)
     Output: AX = Status
 	(All other registers are preserved)
     */
-    pVbe->pInt10->num = 0x10;
-    pVbe->pInt10->ax = 0x4f02;
-    pVbe->pInt10->bx = mode;
+
+    xf86Int10InfoPtr pInt10 = pVbe->pInt10;
+
+    pInt10->num = 0x10;
+    pInt10->ax = 0x4f02;
+    pInt10->bx = mode & ~(1 << 11);
     if (block) {
-	pVbe->pInt10->bx |= 1 << 11;
-	memcpy(pVbe->memory, block, sizeof(VbeCRTCInfoBlock));
-	pVbe->pInt10->es = SEG_ADDR(pVbe->real_mode_base);
-	pVbe->pInt10->di = SEG_OFF(pVbe->real_mode_base);
+	CARD8 *pBlock = pVbe->memory;
+
+	pInt10->bx |= 1 << 11;
+	pInt10->es = SEG_ADDR(pVbe->real_mode_base);
+	pInt10->di = SEG_OFF(pVbe->real_mode_base);
+
+	/*
+	 * Move most fields byte by byte to avoid endianness and alignment
+	 * issues.
+	 */
+	pBlock[0] = block->HorizontalTotal;
+	pBlock[1] = block->HorizontalTotal >> 8;
+	pBlock[2] = block->HorizontalSyncStart;
+	pBlock[3] = block->HorizontalSyncStart >> 8;
+	pBlock[4] = block->HorizontalSyncEnd;
+	pBlock[5] = block->HorizontalSyncEnd >> 8;
+	pBlock[6] = block->VerticalTotal;
+	pBlock[7] = block->VerticalTotal >> 8;
+	pBlock[8] = block->VerticalSyncStart;
+	pBlock[9] = block->VerticalSyncStart >> 8;
+	pBlock[10] = block->VerticalSyncEnd;
+	pBlock[11] = block->VerticalSyncEnd >> 8;
+	pBlock[12] = block->Flags;
+	pBlock[13] = block->PixelClock;
+	pBlock[14] = block->PixelClock >> 8;
+	pBlock[15] = block->PixelClock >> 16;
+	pBlock[16] = block->PixelClock >> 24;
+	pBlock[17] = block->RefreshRate;
+	pBlock[18] = block->RefreshRate >> 8;
+
+	(void) memcpy(pBlock + 19, block->Reserved, 40);
     }
 
-    xf86ExecX86int10(pVbe->pInt10);
+    xf86ExecX86int10(pInt10);
 
-    return (R16(pVbe->pInt10->ax) == 0x4f);
+    return (R16(pInt10->ax) == 0x4f);
 }
 
 Bool
@@ -545,39 +607,40 @@ VBEGetVBEMode(vbeInfoPtr pVbe, int *mode)
 	BX := Current video mode
 	(All other registers are preserved)
     */
+
     pVbe->pInt10->num = 0x10;
     pVbe->pInt10->ax = 0x4f03;
 
     xf86ExecX86int10(pVbe->pInt10);
 
-    if (R16(pVbe->pInt10->ax) == 0x4f) {
-	*mode = R16(pVbe->pInt10->bx);
+    if (R16(pVbe->pInt10->ax) != 0x4f)
+	return FALSE;
 
-	return (TRUE);
-    }
-
-    return (FALSE);
+    *mode = R16(pVbe->pInt10->bx);
+    return TRUE;
 }
 
 VbeModeInfoBlock *
 VBEGetModeInfo(vbeInfoPtr pVbe, int mode)
 {
-    VbeModeInfoBlock *block = NULL;
-
-    bzero(pVbe->memory, sizeof(VbeModeInfoBlock));
-
     /*
     Input:
 	AH    := 4Fh	Super VGA support
 	AL    := 01h	Return Super VGA mode information
-	CX    := 	Super VGA video mode
-			(mode number must be one of those returned by Function 0)
+	CX    :=	Super VGA video mode
+			(mode number must be one returned by Function 0)
 	ES:DI := Pointer to buffer
 
     Output:
 	AX    := status
 	(All other registers are preserved)
      */
+
+    VbeModeInfoBlock *block;
+    char *pBlock = pVbe->memory;
+
+    (void) memset(pBlock, 0, 256);
+
     pVbe->pInt10->num = 0x10;
     pVbe->pInt10->ax = 0x4f01;
     pVbe->pInt10->cx = mode;
@@ -585,73 +648,80 @@ VBEGetModeInfo(vbeInfoPtr pVbe, int mode)
     pVbe->pInt10->di = SEG_OFF(pVbe->real_mode_base);
     xf86ExecX86int10(pVbe->pInt10);
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (NULL);
+	return NULL;
 
-    block = xcalloc(sizeof(VbeModeInfoBlock), 1);
+    block = xnfcalloc(sizeof(VbeModeInfoBlock), 1);
 
-    block->ModeAttributes = *(CARD16*)pVbe->memory;
-    block->WinAAttributes = ((char*)pVbe->memory)[2];
-    block->WinBAttributes = ((char*)pVbe->memory)[3];
-    block->WinGranularity = *(CARD16*)(((char*)pVbe->memory) + 4);
-    block->WinSize = *(CARD16*)(((char*)pVbe->memory) + 6);
-    block->WinASegment = *(CARD16*)(((char*)pVbe->memory) + 8);
-    block->WinBSegment = *(CARD16*)(((char*)pVbe->memory) + 10);
-    block->WinFuncPtr = *(CARD32*)(((char*)pVbe->memory) + 12);
-    block->BytesPerScanline = *(CARD16*)(((char*)pVbe->memory) + 16);
+    block->ModeAttributes = B_O16(pBlock[0]);
+    block->WinAAttributes = pBlock[2];
+    block->WinBAttributes = pBlock[3];
+    block->WinGranularity = B_O16(pBlock[4]);
+    block->WinSize = B_O16(pBlock[6]);
+    block->WinASegment = B_O16(pBlock[8]);
+    block->WinBSegment = B_O16(pBlock[10]);
+    block->WinFuncPtr = B_O32(pBlock[12]);
+    block->BytesPerScanline = B_O16(pBlock[16]);
+
+    if (pVbe->version < 0x0102) {
+	(void) memcpy(&block->Reserved2, pBlock + 18, 238);
+	return block;
+    }
 
     /* mandatory information for VBE 1.2 and above */
-    block->XResolution = *(CARD16*)(((char*)pVbe->memory) + 18);
-    block->YResolution = *(CARD16*)(((char*)pVbe->memory) + 20);
-    block->XCharSize = ((char*)pVbe->memory)[22];
-    block->YCharSize = ((char*)pVbe->memory)[23];
-    block->NumberOfPlanes = ((char*)pVbe->memory)[24];
-    block->BitsPerPixel = ((char*)pVbe->memory)[25];
-    block->NumberOfBanks = ((char*)pVbe->memory)[26];
-    block->MemoryModel = ((char*)pVbe->memory)[27];
-    block->BankSize = ((char*)pVbe->memory)[28];
-    block->NumberOfImages = ((char*)pVbe->memory)[29];
-    block->Reserved = ((char*)pVbe->memory)[30];
+    block->XResolution = B_O16(pBlock[18]);
+    block->YResolution = B_O16(pBlock[20]);
+    block->XCharSize = pBlock[22];
+    block->YCharSize = pBlock[23];
+    block->NumberOfPlanes = pBlock[24];
+    block->BitsPerPixel = pBlock[25];
+    block->NumberOfBanks = pBlock[26];
+    block->MemoryModel = pBlock[27];
+    block->BankSize = pBlock[28];
+    block->NumberOfImages = pBlock[29];
+    block->Reserved = pBlock[30];
 
     /* Direct color fields (required for direct/6 and YUV/7 memory models) */
-    block->RedMaskSize = ((char*)pVbe->memory)[31];
-    block->RedFieldPosition = ((char*)pVbe->memory)[32];
-    block->GreenMaskSize = ((char*)pVbe->memory)[33];
-    block->GreenFieldPosition = ((char*)pVbe->memory)[34];
-    block->BlueMaskSize = ((char*)pVbe->memory)[35];
-    block->BlueFieldPosition = ((char*)pVbe->memory)[36];
-    block->RsvdMaskSize = ((char*)pVbe->memory)[37];
-    block->RsvdFieldPosition = ((char*)pVbe->memory)[38];
-    block->DirectColorModeInfo = ((char*)pVbe->memory)[39];
+    block->RedMaskSize = pBlock[31];
+    block->RedFieldPosition = pBlock[32];
+    block->GreenMaskSize = pBlock[33];
+    block->GreenFieldPosition = pBlock[34];
+    block->BlueMaskSize = pBlock[35];
+    block->BlueFieldPosition = pBlock[36];
+    block->RsvdMaskSize = pBlock[37];
+    block->RsvdFieldPosition = pBlock[38];
+    block->DirectColorModeInfo = pBlock[39];
+
+    if (pVbe->version < 0x0200) {
+	(void) memcpy(&block->Reserved2, pBlock + 40, 216);
+	return block;
+    }
 
     /* Mandatory information for VBE 2.0 and above */
-    if (pVbe->version >= 0x200) {
-	block->PhysBasePtr = *(CARD32*)(((char*)pVbe->memory) + 40);
-	block->Reserved32 = *(CARD32*)(((char*)pVbe->memory) + 44);
-	block->Reserved16 = *(CARD16*)(((char*)pVbe->memory) + 48);
+    block->PhysBasePtr = B_O32((pBlock[40]));
+    block->Reserved32 = B_O32(pBlock[44]);
+    block->Reserved16 = B_O16(pBlock[48]);
 
-	/* Mandatory information for VBE 3.0 and above */
-	if (pVbe->version >= 0x300) {
-	    block->LinBytesPerScanLine = *(CARD16*)(((char*)pVbe->memory) + 50);
-	    block->BnkNumberOfImagePages = ((char*)pVbe->memory)[52];
-	    block->LinNumberOfImagePages = ((char*)pVbe->memory)[53];
-	    block->LinRedMaskSize = ((char*)pVbe->memory)[54];
-	    block->LinRedFieldPosition = ((char*)pVbe->memory)[55];
-	    block->LinGreenMaskSize = ((char*)pVbe->memory)[56];
-	    block->LinGreenFieldPosition = ((char*)pVbe->memory)[57];
-	    block->LinBlueMaskSize = ((char*)pVbe->memory)[58];
-	    block->LinBlueFieldPosition = ((char*)pVbe->memory)[59];
-	    block->LinRsvdMaskSize = ((char*)pVbe->memory)[60];
-	    block->LinRsvdFieldPosition = ((char*)pVbe->memory)[61];
-	    block->MaxPixelClock = *(CARD32*)(((char*)pVbe->memory) + 62);
-	    memcpy(&block->Reserved2, ((char*)pVbe->memory) + 66, 188);
-	}
-	else
-	memcpy(&block->LinBytesPerScanLine, ((char*)pVbe->memory) + 50, 206);
+    if (pVbe->version < 0x0300) {
+	(void) memcpy(&block->Reserved2, pBlock + 50, 206);
+	return block;
     }
-    else
-	memcpy(&block->PhysBasePtr, ((char*)pVbe->memory) + 40, 216);
 
-    return (block);
+    /* Mandatory information for VBE 3.0 and above */
+    block->LinBytesPerScanLine = B_O16(pBlock[50]);
+    block->BnkNumberOfImagePages = pBlock[52];
+    block->LinNumberOfImagePages = pBlock[53];
+    block->LinRedMaskSize = pBlock[54];
+    block->LinRedFieldPosition = pBlock[55];
+    block->LinGreenMaskSize = pBlock[56];
+    block->LinGreenFieldPosition = pBlock[57];
+    block->LinBlueMaskSize = pBlock[58];
+    block->LinBlueFieldPosition = pBlock[59];
+    block->LinRsvdMaskSize = pBlock[60];
+    block->LinRsvdFieldPosition = pBlock[61];
+    block->MaxPixelClock = B_O32(pBlock[62]);
+    (void) memcpy(&block->Reserved2, pBlock + 66, 190);
+
+    return block;
 }
 
 void
@@ -661,7 +731,7 @@ VBEFreeModeInfo(VbeModeInfoBlock *block)
 }
 
 Bool
-VBESaveRestore(vbeInfoPtr pVbe, vbeSaveRestoreFunction function, 
+VBESaveRestore(vbeInfoPtr pVbe, vbeSaveRestoreFunction function,
 	       pointer *memory, int *size, int *real_mode_pages)
 {
     /*
@@ -680,7 +750,6 @@ VBESaveRestore(vbeInfoPtr pVbe, vbeSaveRestoreFunction function,
 	BX = Number of 64-byte blocks to hold the state buffer
 	(All other registers are preserved)
 
-
     Input:
 	AH    := 4Fh	Super VGA support
 	AL    := 04h	Save/restore Super VGA video state
@@ -692,7 +761,6 @@ VBESaveRestore(vbeInfoPtr pVbe, vbeSaveRestoreFunction function,
 	AX    := Status
 	(All other registers are preserved)
 
-
     Input:
 	AH    := 4Fh	Super VGA support
 	AL    := 04h	Save/restore Super VGA video state
@@ -705,58 +773,65 @@ VBESaveRestore(vbeInfoPtr pVbe, vbeSaveRestoreFunction function,
 	(All other registers are preserved)
      */
 
-    if ((pVbe->version & 0xff00) > 0x100) {
-        int screen = pVbe->pInt10->scrnIndex;
-        if (function == MODE_QUERY ||
-	    (function == MODE_SAVE && !*memory)) {
-	    /* Query amount of memory to save state */
+    xf86Int10InfoPtr pInt10;
 
-	    pVbe->pInt10->num = 0x10;
-	    pVbe->pInt10->ax = 0x4f04;
-	    pVbe->pInt10->dx = 0;
-	    pVbe->pInt10->cx = 0x000f;
-	    xf86ExecX86int10(pVbe->pInt10);
-	    if (R16(pVbe->pInt10->ax) != 0x4f)
-	        return (FALSE);
+    /* Earlier VBE versions won't give us an error indication */
+    if (pVbe->version < 0x0200)
+	return TRUE;
 
-	    if (function == MODE_SAVE) {
-	        int npages = (R16(pVbe->pInt10->bx) * 64) / 4096 + 1;
-		if ((*memory = xf86Int10AllocPages(pVbe->pInt10, npages,
-						   real_mode_pages)) == NULL) {
-		    xf86DrvMsg(screen, X_ERROR,
-			       "Cannot allocate memory to save SVGA state.\n");
-		    return (FALSE);
-		}
+    pInt10 = pVbe->pInt10;
+
+    if ((function == MODE_QUERY) || ((function == MODE_SAVE) && !*memory)) {
+	/* Query amount of memory to save state */
+
+	pInt10->num = 0x10;
+	pInt10->ax = 0x4f04;
+	pInt10->dx = 0;
+	pInt10->cx = 0x000f;
+	xf86ExecX86int10(pInt10);
+	if (R16(pInt10->ax) != 0x4f)
+	    return FALSE;
+
+	if (function == MODE_SAVE) {
+	    int npages = (R16(pInt10->bx) * 64) / 4096 + 1;
+
+	    *memory = xf86Int10AllocPages(pInt10, npages, real_mode_pages);
+	    if (!*memory) {
+		xf86DrvMsg(pInt10->scrnIndex, X_ERROR,
+			   "Cannot allocate memory to save SVGA state.\n");
+		return FALSE;
 	    }
-	    *size = pVbe->pInt10->bx * 64;
 	}
 
-	/* Save/Restore Super VGA state */
-	if (function != MODE_QUERY) {
-	    
-	    if (!*memory) return FALSE;
-	    pVbe->pInt10->num = 0x10;
-	    pVbe->pInt10->ax = 0x4f04;
-	    switch (function) {
-	    case MODE_SAVE:
-	      pVbe->pInt10->dx = 1;
-	      break;
-	    case MODE_RESTORE:
-	      pVbe->pInt10->dx = 2;
-	      break;
-	    case MODE_QUERY:
-	      return FALSE;
-	    }
-	    pVbe->pInt10->cx = 0x000f;
-	    
-	    pVbe->pInt10->es = SEG_ADDR(*real_mode_pages);
-	    pVbe->pInt10->bx = SEG_OFF(*real_mode_pages);
-	    xf86ExecX86int10(pVbe->pInt10);
-	    return (R16(pVbe->pInt10->ax) == 0x4f);
+	*size = pInt10->bx * 64;
 
-	}
+	if (function == MODE_QUERY)
+	    return TRUE;
     }
-    return TRUE;
+
+    /* Save/Restore Super VGA state */
+    if (!*memory)
+	return FALSE;
+
+    pInt10->num = 0x10;
+    pInt10->ax = 0x4f04;
+    pInt10->cx = 0x000f;
+    pInt10->es = SEG_ADDR(*real_mode_pages);
+    pInt10->bx = SEG_OFF(*real_mode_pages);
+
+    switch (function) {
+    case MODE_SAVE:
+	pInt10->dx = 1;
+	break;
+    case MODE_RESTORE:
+	pInt10->dx = 2;
+	break;
+    default:
+	return FALSE;
+    }
+
+    xf86ExecX86int10(pInt10);
+    return (R16(pInt10->ax) == 0x4f);
 }
 
 Bool
@@ -766,28 +841,30 @@ VBEBankSwitch(vbeInfoPtr pVbe, unsigned int iBank, int window)
     Input:
 	AH    := 4Fh	Super VGA support
 	AL    := 05h
+	BH    := 00h	Set memory window
+	      := 01h	Get memory window
+	BL    := 00h	Window A
+	      := 01h	Window B
+	DX    := Window number to set
 
     Output:
+	AX    := Status
+	DX    := Window number retrieved
      */
+
     pVbe->pInt10->num = 0x10;
     pVbe->pInt10->ax = 0x4f05;
     pVbe->pInt10->bx = window;
     pVbe->pInt10->dx = iBank;
     xf86ExecX86int10(pVbe->pInt10);
 
-    if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (FALSE);
-
-    return (TRUE);
+    return (R16(pVbe->pInt10->ax) == 0x4f);
 }
 
 Bool
 VBESetGetLogicalScanlineLength(vbeInfoPtr pVbe, vbeScanwidthCommand command,
 				int width, int *pixels, int *bytes, int *max)
 {
-    if (command < SCANWID_SET || command > SCANWID_GET_MAX)
-	return (FALSE);
-
     /*
     Input:
 	AX := 4F06h VBE Set/Get Logical Scan Line Length
@@ -807,6 +884,9 @@ VBESetGetLogicalScanlineLength(vbeInfoPtr pVbe, vbeScanwidthCommand command,
 	DX := Maximum Number of Scan Lines
      */
 
+    if (command < SCANWID_SET || command > SCANWID_GET_MAX)
+	return FALSE;
+
     pVbe->pInt10->num = 0x10;
     pVbe->pInt10->ax = 0x4f06;
     pVbe->pInt10->bx = command;
@@ -815,7 +895,7 @@ VBESetGetLogicalScanlineLength(vbeInfoPtr pVbe, vbeScanwidthCommand command,
     xf86ExecX86int10(pVbe->pInt10);
 
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (FALSE);
+	return FALSE;
 
     if (command == SCANWID_GET || command == SCANWID_GET_MAX) {
 	if (pixels)
@@ -826,7 +906,7 @@ VBESetGetLogicalScanlineLength(vbeInfoPtr pVbe, vbeScanwidthCommand command,
 	    *max = R16(pVbe->pInt10->dx);
     }
 
-    return (TRUE);
+    return TRUE;
 }
 
 Bool
@@ -839,10 +919,7 @@ VBESetDisplayStart(vbeInfoPtr pVbe, int x, int y, Bool wait_retrace)
     pVbe->pInt10->dx = y;
     xf86ExecX86int10(pVbe->pInt10);
 
-    if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (FALSE);
-
-    return (TRUE);
+    return (R16(pVbe->pInt10->ax) == 0x4f);
 }
 
 Bool
@@ -854,12 +931,12 @@ VBEGetDisplayStart(vbeInfoPtr pVbe, int *x, int *y)
     xf86ExecX86int10(pVbe->pInt10);
 
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (FALSE);
+	return FALSE;
 
     *x = pVbe->pInt10->cx;
     *y = pVbe->pInt10->dx;
 
-    return (TRUE);
+    return TRUE;
 }
 
 int
@@ -882,12 +959,12 @@ VBESetGetDACPaletteFormat(vbeInfoPtr pVbe, int bits)
     pVbe->pInt10->ax = 0x4f08;
     if (!bits)
 	pVbe->pInt10->bx = 0x01;
-    else 
+    else
 	pVbe->pInt10->bx = (bits & 0x00ff) << 8;
     xf86ExecX86int10(pVbe->pInt10);
 
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (0);
+	return 0;
 
     return (bits != 0 ? bits : (pVbe->pInt10->bx >> 8) & 0x00ff);
 }
@@ -898,7 +975,6 @@ VBESetGetPaletteData(vbeInfoPtr pVbe, Bool set, int first, int num,
 {
     /*
     Input:
-    (16-bit)
 	AX    := 4F09h VBE Load/Unload Palette Data
 	BL    := 00h Set Palette Data
 	      := 01h Get Palette Data
@@ -911,16 +987,6 @@ VBESetGetPaletteData(vbeInfoPtr pVbe, Bool set, int first, int num,
 
     Output:
 	AX    := VBE Return Status
-
-
-    Input:
-    (32-bit)
-	BL     := 00h Set Palette Data
-	       := 80h Set Palette Data during Vertical Retrace
-	CX     := Number of palette registers to update (to a maximum of 256)
-	DX     := First of the palette registers to update (start)
-	ES:EDI := Table of palette values (see below for format)
-	DS     := Selector for memory mapped registers
      */
 
     pVbe->pInt10->num = 0x10;
@@ -934,26 +1000,24 @@ VBESetGetPaletteData(vbeInfoPtr pVbe, Bool set, int first, int num,
     pVbe->pInt10->es = SEG_ADDR(pVbe->real_mode_base);
     pVbe->pInt10->di = SEG_OFF(pVbe->real_mode_base);
     if (set)
-	memcpy(pVbe->memory, data, num * sizeof(CARD32));
+	(void) memcpy(pVbe->memory, data, num * sizeof(CARD32));
     xf86ExecX86int10(pVbe->pInt10);
 
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (NULL);
+	return NULL;
 
     if (set)
-	return (data);
+	return data;
 
-    data = xalloc(num * sizeof(CARD32));
-    memcpy(data, pVbe->memory, num * sizeof(CARD32));
+    data = xnfalloc(num * sizeof(CARD32));
+    (void) memcpy(data, pVbe->memory, num * sizeof(CARD32));
 
-    return (data);
+    return data;
 }
 
 VBEpmi *
 VBEGetVBEpmi(vbeInfoPtr pVbe)
 {
-    VBEpmi *pmi;
-
     /*
     Input:
 	AH    := 4Fh	Super VGA support
@@ -964,10 +1028,13 @@ VBEGetVBEpmi(vbeInfoPtr pVbe)
 	AX    := Status
 	ES    := Real Mode Segment of Table
 	DI    := Offset of Table
-	CX    := Lenght of Table including protected mode code in bytes (for copying purposes)
+	CX    := Length of Table including protected mode code in bytes
+		 (for copying purposes)
 	(All other registers are preserved)
      */
 
+    VBEpmi *pmi;
+
     pVbe->pInt10->num = 0x10;
     pVbe->pInt10->ax = 0x4f0a;
     pVbe->pInt10->bx = 0;
@@ -975,100 +1042,14 @@ VBEGetVBEpmi(vbeInfoPtr pVbe)
     xf86ExecX86int10(pVbe->pInt10);
 
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (NULL);
+	return NULL;
 
-    pmi = xalloc(sizeof(VBEpmi));
+    pmi = xnfalloc(sizeof(VBEpmi));
     pmi->seg_tbl = R16(pVbe->pInt10->es);
     pmi->tbl_off = R16(pVbe->pInt10->di);
     pmi->tbl_len = R16(pVbe->pInt10->cx);
 
-    return (pmi);
-}
-
-#if 0
-vbeModeInfoPtr
-VBEBuildVbeModeList(vbeInfoPtr pVbe, VbeInfoBlock *vbe)
-{
-    vbeModeInfoPtr ModeList = NULL;
-
-    int i = 0;
-    while (vbe->VideoModePtr[i] != 0xffff) {
-	vbeModeInfoPtr m;
-	VbeModeInfoBlock *mode;
-	int id = vbe->VideoModePtr[i++];
-	int bpp;
-
-	if ((mode = VBEGetModeInfo(pVbe, id)) == NULL)
-	    continue;
-
- 	bpp = mode->BitsPerPixel;
-
-	m = xnfcalloc(sizeof(vbeModeInfoRec),1);
-	m->width = mode->XResolution;
-	m->height = mode->YResolution;
-	m->bpp = bpp;
-	m->n = id;
-	m->next = ModeList;
-
-	xf86DrvMsgVerb(pVbe->pInt10->scrnIndex, X_PROBED, 3,
-		       "BIOS reported VESA mode 0x%x: x:%i y:%i bpp:%i\n",
-		       m->n, m->width, m->height, m->bpp);
-
-	ModeList = m;
-
-	VBEFreeModeInfo(mode);
-    }
-    return ModeList;
-}
-
-unsigned short 
-VBECalcVbeModeIndex(vbeModeInfoPtr m, DisplayModePtr mode, int bpp)
-{
-    while (m) {
-	if (bpp == m->bpp 
-	    && mode->HDisplay == m->width 
-	    && mode->VDisplay == m->height)
-	    return m->n;
-	m = m->next;
-    }
-    return 0;
-}
-#endif
-
-void
-VBEVesaSaveRestore(vbeInfoPtr pVbe, vbeSaveRestorePtr vbe_sr,
-		  vbeSaveRestoreFunction function)
-{
-    Bool SaveSucc = FALSE;
-
-    if (VBE_VERSION_MAJOR(pVbe->version) > 1
-	&& (function == MODE_SAVE || vbe_sr->pstate)) {
-	if (function == MODE_RESTORE)
-	    memcpy(vbe_sr->state, vbe_sr->pstate, vbe_sr->stateSize);
-	ErrorF("VBESaveRestore\n");
-	if ((VBESaveRestore(pVbe,function,
-			    (pointer)&vbe_sr->state,
-			    &vbe_sr->stateSize,&vbe_sr->statePage))) {
-	    if (function == MODE_SAVE) {
-		SaveSucc = TRUE;
-		vbe_sr->stateMode = -1; /* invalidate */
-		/* don't rely on the memory not being touched */
-		if (vbe_sr->pstate == NULL)
-		    vbe_sr->pstate = xalloc(vbe_sr->stateSize);
-		memcpy(vbe_sr->pstate, vbe_sr->state, vbe_sr->stateSize);
-	    }
-	    ErrorF("VBESaveRestore done with success\n");
-	    return;
-	}
-	ErrorF("VBESaveRestore done\n");
-    } 
-    
-    if (function == MODE_SAVE && !SaveSucc)
-	    (void)VBEGetVBEMode(pVbe, &vbe_sr->stateMode);
-	
-    if (function == MODE_RESTORE && vbe_sr->stateMode != -1)
-	    VBESetVBEMode(pVbe, vbe_sr->stateMode, NULL);
-
+    return pmi;
 }
 
 int
@@ -1079,8 +1060,8 @@ VBEGetPixelClock(vbeInfoPtr pVbe, int mode, int clock)
 	AX := 4F0Bh VBE Get Pixel Clock
 	BL := 01h Get Pixel Clock
 	ECX := pixel clock in units of Hz
-        DX := mode number
-     
+	DX := mode number
+
     Output:
 	AX := VBE Return Status
 	ECX := Closest pixel clock
@@ -1094,9 +1075,9 @@ VBEGetPixelClock(vbeInfoPtr pVbe, int mode, int clock)
     xf86ExecX86int10(pVbe->pInt10);
 
     if (R16(pVbe->pInt10->ax) != 0x4f)
-	return (0);
+	return 0;
 
-    return (pVbe->pInt10->cx);
+    return pVbe->pInt10->cx;
 }
 
 Bool
@@ -1107,7 +1088,7 @@ VBEDPMSSet(vbeInfoPtr pVbe, int mode)
 	AX := 4F10h DPMS
 	BL := 01h Set Display Power State
 	BH := requested power state
-     
+
     Output:
 	AX := VBE Return Status
      */
