@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/twm/util.c,v 1.17 2007/10/10 15:23:02 tsi Exp $ */
 /*****************************************************************************/
 /*
 
@@ -48,12 +49,9 @@ in this Software without prior written authorization from The Open Group.
 /**    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE    **/
 /**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
 /*****************************************************************************/
-/* $XFree86: xc/programs/twm/util.c,v 1.14 2004/06/08 01:17:02 dawes Exp $ */
 
 
 /***********************************************************************
- *
- * $Xorg: util.c,v 1.5 2001/02/09 02:05:37 xorgcvs Exp $
  *
  * utility routines for twm
  *
@@ -72,15 +70,15 @@ in this Software without prior written authorization from The Open Group.
 #include <X11/Xmu/CharSet.h>
 #include <X11/xpm.h>
 
-static Pixmap CreateXLogoPixmap ( unsigned int *widthp, 
+static Pixmap CreateXLogoPixmap ( unsigned int *widthp,
 				  unsigned int *heightp );
-static Pixmap CreateResizePixmap ( unsigned int *widthp, 
+static Pixmap CreateResizePixmap ( unsigned int *widthp,
 				   unsigned int *heightp );
-static Pixmap CreateDotPixmap ( unsigned int *widthp, 
+static Pixmap CreateDotPixmap ( unsigned int *widthp,
 				unsigned int *heightp );
-static Pixmap CreateQuestionPixmap ( unsigned int *widthp, 
+static Pixmap CreateQuestionPixmap ( unsigned int *widthp,
 				     unsigned int *heightp );
-static Pixmap CreateMenuPixmap ( unsigned int *widthp, 
+static Pixmap CreateMenuPixmap ( unsigned int *widthp,
 				 unsigned int *heightp );
 
 int HotX, HotY;
@@ -103,9 +101,8 @@ int HotX, HotY;
  */
 
 /* ARGSUSED */
-void MoveOutline(root, x, y, width, height, bw, th)
-    Window root;
-    int x, y, width, height, bw, th;
+void
+MoveOutline(Window root, int x, int y, int width, int height, int bw, int th)
 {
     static int	lastx = 0;
     static int	lasty = 0;
@@ -116,12 +113,12 @@ void MoveOutline(root, x, y, width, height, bw, th)
     int		xl, xr, yt, yb, xinnerl, xinnerr, yinnert, yinnerb;
     int		xthird, ythird;
     XSegment	outline[18];
-    register XSegment	*r;
+    XSegment	*r;
 
     if (x == lastx && y == lasty && width == lastWidth && height == lastHeight
 	&& lastBW == bw && th == lastTH)
 	return;
-    
+
     r = outline;
 
 #define DRAWIT() \
@@ -230,8 +227,7 @@ void MoveOutline(root, x, y, width, height, bw, th)
  */
 
 void
-Zoom(wf, wt)
-    Window wf, wt;
+Zoom(Window wf, Window wt)
 {
     int fx, fy, tx, ty;			/* from, to */
     unsigned int fw, fh, tw, th;	/* from, to */
@@ -261,7 +257,7 @@ Zoom(wf, wt)
 	    int y = fy + (int) ((dy * i) / z);
 	    unsigned width = (unsigned) (((long) fw) + (dw * i) / z);
 	    unsigned height = (unsigned) (((long) fh) + (dh * i) / z);
-	
+
 	    XDrawRectangle (dpy, Scr->Root, Scr->DrawGC,
 			    x, y, width, height);
 	}
@@ -286,8 +282,7 @@ Zoom(wf, wt)
  */
 
 char *
-ExpandFilename(name)
-char *name;
+ExpandFilename(char *name)
 {
     char *newname;
 
@@ -295,7 +290,7 @@ char *name;
 
     newname = (char *) malloc (HomeLen + strlen(name) + 2);
     if (!newname) {
-	fprintf (stderr, 
+	fprintf (stderr,
 		 "%s:  unable to allocate %ld bytes to expand filename %s/%s\n",
 		 ProgramName, HomeLen + (unsigned long)strlen(name) + 2,
 		 Home, &name[1]);
@@ -318,8 +313,7 @@ char *name;
  */
 
 void
-GetUnknownIcon(name)
-char *name;
+GetUnknownIcon(char *name)
 {
     if ((Scr->UnknownPm = GetBitmap(name)) != None)
     {
@@ -344,10 +338,8 @@ char *name;
  ***********************************************************************
  */
 
-Pixmap 
-FindBitmap (name, widthp, heightp)
-    char *name;
-    unsigned int *widthp, *heightp;
+Pixmap
+FindBitmap (char *name, unsigned int *widthp, unsigned int *heightp)
 {
     char *bigname;
     Pixmap pm;
@@ -373,7 +365,7 @@ FindBitmap (name, widthp, heightp)
 	    { TBPM_MENU,	CreateMenuPixmap },
 	    { TBPM_QUESTION,	CreateQuestionPixmap },
 	};
-	
+
 	for (i = 0; i < (sizeof pmtab)/(sizeof pmtab[0]); i++) {
 	    if (XmuCompareISOLatin1 (pmtab[i].name, name) == 0)
 	      return (*pmtab[i].proc) (widthp, heightp);
@@ -448,26 +440,21 @@ FindBitmap (name, widthp, heightp)
     }
     if (bigname != name) free (bigname);
     if (pm == None) {
-	fprintf (stderr, "%s:  unable to find bitmap \"%s\"\n", 
+	fprintf (stderr, "%s:  unable to find bitmap \"%s\"\n",
 		 ProgramName, name);
     }
 
     return pm;
 }
 
-Pixmap 
-GetBitmap (name)
-    char *name;
+Pixmap
+GetBitmap (char *name)
 {
     return FindBitmap (name, &JunkWidth, &JunkHeight);
 }
 
 void
-InsertRGBColormap (a, maps, nmaps, replace)
-    Atom a;
-    XStandardColormap *maps;
-    int nmaps;
-    Bool replace;
+InsertRGBColormap (Atom a, XStandardColormap *maps, int nmaps, Bool replace)
 {
     StdCmap *sc = NULL;
 
@@ -501,18 +488,15 @@ InsertRGBColormap (a, maps, nmaps, replace)
     }
     sc->nmaps = nmaps;
     sc->maps = maps;
-
-    return;
 }
 
 void
-RemoveRGBColormap (a)
-    Atom a;
+RemoveRGBColormap (Atom a)
 {
     StdCmap *sc, *prev;
 
     prev = NULL;
-    for (sc = Scr->StdCmapInfo.head; sc; sc = sc->next) {  
+    for (sc = Scr->StdCmapInfo.head; sc; sc = sc->next) {
 	if (sc->atom == a) break;
 	prev = sc;
     }
@@ -523,11 +507,10 @@ RemoveRGBColormap (a)
 	if (Scr->StdCmapInfo.tail == sc) Scr->StdCmapInfo.tail = prev;
 	if (Scr->StdCmapInfo.mru == sc) Scr->StdCmapInfo.mru = NULL;
     }
-    return;
 }
 
 void
-LocateStandardColormaps()
+LocateStandardColormaps(void)
 {
     Atom *atoms;
     int natoms;
@@ -544,14 +527,10 @@ LocateStandardColormaps()
 	}
     }
     if (atoms) XFree ((char *) atoms);
-    return;
 }
 
 void
-GetColor(kind, what, name)
-int kind;
-Pixel *what;
-char *name;
+GetColor(int kind, Pixel *what, char *name)
 {
     XColor color, junkcolor;
     Status stat = 0;
@@ -577,7 +556,7 @@ char *name;
 	    stat = XParseColor (dpy, cmap, name, &color);
 	if (!stat)
 	{
-	    fprintf (stderr, "%s:  invalid color name \"%s\"\n", 
+	    fprintf (stderr, "%s:  invalid color name \"%s\"\n",
 		     ProgramName, name);
 	    return;
 	}
@@ -608,7 +587,7 @@ char *name;
 
       gotit:
 	if (stdcmap) {
-            color.pixel = (stdcmap->base_pixel +
+	    color.pixel = (stdcmap->base_pixel +
 			   ((Pixel)(((float)color.red / 65535.0) *
 				    stdcmap->red_max + 0.5) *
 			    stdcmap->red_mult) +
@@ -618,8 +597,8 @@ char *name;
 			   ((Pixel)(((float)color.blue  / 65535.0) *
 				    stdcmap->blue_max + 0.5) *
 			    stdcmap->blue_mult));
-        } else {
-	    fprintf (stderr, "%s:  unable to allocate color \"%s\"\n", 
+	} else {
+	    fprintf (stderr, "%s:  unable to allocate color \"%s\"\n",
 		     ProgramName, name);
 	    return;
 	}
@@ -629,10 +608,7 @@ char *name;
 }
 
 void
-GetColorValue(kind, what, name)
-int kind;
-XColor *what;
-char *name;
+GetColorValue(int kind, XColor *what, char *name)
 {
     XColor junkcolor;
     Colormap cmap = Scr->TwmRoot.cmaps.cwins[0]->colormap->c;
@@ -647,7 +623,7 @@ char *name;
 
     if (!XLookupColor (dpy, cmap, name, what, &junkcolor))
     {
-	fprintf (stderr, "%s:  invalid color name \"%s\"\n", 
+	fprintf (stderr, "%s:  invalid color name \"%s\"\n",
 		 ProgramName, name);
     }
     else
@@ -656,7 +632,7 @@ char *name;
     }
 }
 
-/* 
+/*
  * The following functions are sensible to 'use_fontset'.
  * When 'use_fontset' is True,
  *  - XFontSet-related internationalized functions are used
@@ -667,8 +643,7 @@ char *name;
  *     locale is not set properly.
  */
 void
-GetFont(font)
-MyFont *font;
+GetFont(MyFont *font)
 {
     char *deffontname = "fixed";
     char **missing_charset_list_return;
@@ -677,7 +652,7 @@ MyFont *font;
     XFontSetExtents *font_extents;
     XFontStruct **xfonts;
     char **font_names;
-    register int i;
+    int i;
     int ascent;
     int descent;
     int fnum;
@@ -742,10 +717,7 @@ MyFont *font;
 }
 
 int
-MyFont_TextWidth(font, string, len)
-    MyFont *font;
-    char *string;
-    int len;
+MyFont_TextWidth(MyFont *font, char *string, int len)
 {
     XRectangle ink_rect;
     XRectangle logical_rect;
@@ -759,14 +731,8 @@ MyFont_TextWidth(font, string, len)
 }
 
 void
-MyFont_DrawImageString(dpy, d, font, gc, x, y, string, len)
-    Display *dpy;
-    Drawable d;
-    MyFont *font;
-    GC gc;
-    int x,y;
-    char *string;
-    int len;
+MyFont_DrawImageString(Display *dpy, Drawable d, MyFont *font, GC gc,
+		       int x, int y, char *string, int len)
 {
     if (use_fontset) {
 	XmbDrawImageString(dpy, d, font->fontset, gc, x, y, string, len);
@@ -776,14 +742,8 @@ MyFont_DrawImageString(dpy, d, font, gc, x, y, string, len)
 }
 
 void
-MyFont_DrawString(dpy, d, font, gc, x, y, string, len)
-    Display *dpy;
-    Drawable d;
-    MyFont *font;
-    GC gc;
-    int x,y;
-    char *string;
-    int len;
+MyFont_DrawString(Display *dpy, Drawable d, MyFont *font, GC gc, int x, int y,
+		  char *string, int len)
 {
     if (use_fontset) {
 	XmbDrawString(dpy, d, font->fontset, gc, x, y, string, len);
@@ -793,9 +753,8 @@ MyFont_DrawString(dpy, d, font, gc, x, y, string, len)
 }
 
 void
-MyFont_ChangeGC(fix_fore, fix_back, fix_font)
-    unsigned long fix_fore, fix_back;
-    MyFont *fix_font;
+MyFont_ChangeGC(unsigned long fix_fore, unsigned long fix_back,
+		MyFont *fix_font)
 {
     Gcv.foreground = fix_fore;
     Gcv.background = fix_back;
@@ -810,22 +769,19 @@ MyFont_ChangeGC(fix_fore, fix_back, fix_font)
 /*
  * The following functions are internationalized substitutions
  * for XFetchName and XGetIconName using XGetWMName and
- * XGetWMIconName.  
+ * XGetWMIconName.
  *
- * Please note that the third arguments have to be freed using free(), 
+ * Please note that the third arguments have to be freed using free(),
  * not XFree().
  */
 Status
-I18N_FetchName(dpy, w, winname)
-    Display *dpy;
-    Window w;
-    char ** winname;
+I18N_FetchName(Display *dpy, Window w, char ** winname)
 {
     int    status;
     XTextProperty text_prop;
     char **list;
     int    num;
-    
+
     status = XGetWMName(dpy, w, &text_prop);
     if (!status || !text_prop.value || !text_prop.nitems) {
       *winname = NULL;
@@ -833,7 +789,7 @@ I18N_FetchName(dpy, w, winname)
     }
     status = XmbTextPropertyToTextList(dpy, &text_prop, &list, &num);
     if (status < Success || !num || !*list) {
-      *winname = NULL;      
+      *winname = NULL;
       return 0;
     }
     XFree(text_prop.value);
@@ -843,16 +799,13 @@ I18N_FetchName(dpy, w, winname)
 }
 
 Status
-I18N_GetIconName(dpy, w, iconname)
-    Display *dpy;
-    Window w;
-    char ** iconname;
+I18N_GetIconName(Display *dpy, Window w, char ** iconname)
 {
     int    status;
     XTextProperty text_prop;
     char **list;
     int    num;
-	
+
     status = XGetWMIconName(dpy, w, &text_prop);
     if (!status || !text_prop.value || !text_prop.nitems) return 0;
     status = XmbTextPropertyToTextList(dpy, &text_prop, &list, &num);
@@ -868,9 +821,7 @@ I18N_GetIconName(dpy, w, iconname)
  * and easier to debug
  */
 void
-SetFocus (tmp_win, time)
-    TwmWindow *tmp_win;
-    Time	time;
+SetFocus (TwmWindow *tmp_win, Time time)
 {
     Window w = (tmp_win ? tmp_win->w : PointerRoot);
 
@@ -897,8 +848,7 @@ SetFocus (tmp_win, time)
  * called NAME.
  */
 int
-putenv(s)
-    char *s;
+putenv(char *s)
 {
     char *v;
     int varlen, idx;
@@ -924,12 +874,12 @@ putenv(s)
 	    }
 	}
     }
-    
+
     /* add to environment (unless no value; then just return) */
     if(v[1] == 0)
 	return 0;
     if(virgin) {
-	register i;
+	int i;
 
 	newenv = (char **) malloc((unsigned) ((idx + 2) * sizeof(char*)));
 	if(newenv == 0)
@@ -947,15 +897,14 @@ putenv(s)
     environ = newenv;
     environ[idx] = s;
     environ[idx+1] = 0;
-    
+
     return 0;
 }
 #endif /* NOPUTENV */
 
 
-static Pixmap 
-CreateXLogoPixmap (widthp, heightp)
-    unsigned int *widthp, *heightp;
+static Pixmap
+CreateXLogoPixmap (unsigned int *widthp, unsigned int *heightp)
 {
     int h = Scr->TBInfo.width - Scr->TBInfo.border * 2;
     if (h < 0) h = 0;
@@ -989,9 +938,8 @@ CreateXLogoPixmap (widthp, heightp)
 }
 
 
-static Pixmap 
-CreateResizePixmap (widthp, heightp)
-    unsigned int *widthp, *heightp;
+static Pixmap
+CreateResizePixmap (unsigned int *widthp, unsigned int *heightp)
 {
     int h = Scr->TBInfo.width - Scr->TBInfo.border * 2;
     if (h < 1) h = 1;
@@ -1017,7 +965,7 @@ CreateResizePixmap (widthp, heightp)
 	XSetLineAttributes (dpy, gc, lw, LineSolid, CapButt, JoinMiter);
 
 	/*
-	 * draw the resize button, 
+	 * draw the resize button,
 	 */
 	w = (h * 2) / 3;
 	points[0].x = w;
@@ -1045,9 +993,8 @@ CreateResizePixmap (widthp, heightp)
 }
 
 
-static Pixmap 
-CreateDotPixmap (widthp, heightp)
-    unsigned int *widthp, *heightp;
+static Pixmap
+CreateDotPixmap (unsigned int *widthp, unsigned int *heightp)
 {
     int h = Scr->TBInfo.width - Scr->TBInfo.border * 2;
 
@@ -1077,9 +1024,8 @@ CreateDotPixmap (widthp, heightp)
 static char questionmark_bits[] = {
    0x38, 0x7c, 0x64, 0x30, 0x18, 0x00, 0x18, 0x18};
 
-static Pixmap 
-CreateQuestionPixmap (widthp, heightp)
-    unsigned int *widthp, *heightp;
+static Pixmap
+CreateQuestionPixmap (unsigned int *widthp, unsigned int *heightp)
 {
     *widthp = questionmark_width;
     *heightp = questionmark_height;
@@ -1096,18 +1042,15 @@ CreateQuestionPixmap (widthp, heightp)
 }
 
 
-static Pixmap 
-CreateMenuPixmap (widthp, heightp)
-    unsigned int *widthp, *heightp;
+static Pixmap
+CreateMenuPixmap (unsigned int *widthp, unsigned int *heightp)
 {
     return CreateMenuIcon (Scr->TBInfo.width - Scr->TBInfo.border * 2,
 			   widthp,heightp);
 }
 
-Pixmap 
-CreateMenuIcon (height, widthp, heightp)
-    int	height;
-    unsigned int *widthp, *heightp;
+Pixmap
+CreateMenuIcon (int height, unsigned int *widthp, unsigned int *heightp)
 {
     int h, w;
     int ih, iw;
@@ -1181,15 +1124,11 @@ CreateMenuIcon (height, widthp, heightp)
 }
 
 void
-Bell(type,percent,win)
-    int		type;
-    int		percent;
-    Window 	win;
+Bell(int type, int percent, Window win)
 {
 #ifdef XKB
     XkbStdBell(dpy, win, percent, type);
 #else
     XBell(dpy, percent);
 #endif
-    return;
 }
